// ===========Oooo==========================================Oooo========
// =  Copyright (C) 2014 Jinan Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// =====================================================================
//
//  __      __      __
//  \ \    /  \    / /   [File name   ] baudset.v
//   \ \  / /\ \  / /    [Description ] Verilog File for UART Controller Baud Rate Module
//    \ \/ /  \ \/ /     [Timestamp   ] Tue Aug 27 15:30:00 2015
//     \  /    \  /      [version     ] 1.0
//      \/      \/       
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author |Mod. Date |Changes Made:
// V1.0 | XX     |27/08/15  |Initial version
// ===========Oooo==========================================Oooo========


module baudset(
     rst,
     sysclk,
     rxclk
 ); 
 
 input  rst   ;
 input  sysclk; 
 output rxclk ; 

 reg rxclk; 
   
 parameter  fsysclk = 50_000_000; 
 parameter  baud    = 115_200; 
 parameter  divp10x = (10 * fsysclk) / (16 * baud); 
 parameter  divp1x  = fixv(divp10x); 
 parameter  N       = wordsize(divp1x); 
 parameter  divp    = fixv2(divp1x); 
 parameter  low     = 1'b0;
 parameter  hi      = 1'b1; 
        
 reg   [N-1:0] cnt1 = {N{1'b0}}; 
// reg   [3:0]   cnt2 = 4'd0; 
         
//------------function-----------//        
 function [31:0]fixv; 
    input [31:0]value10x;
    fixv = value10x % 10 < 5 ? value10x / 10 : (value10x / 10) + 1;
 endfunction
        
 function [N-1:0]fixv2; 
    input [31:0]value;
    fixv2 = value;
 endfunction
        
 function [5:0]wordsize; 
    input [31:0]value;
      begin
        wordsize = 1'b1;
        while((32'b1 << (wordsize - 1'b1)) < value) wordsize = wordsize + 1'b1;
      end
 endfunction
//---------------------------------//        
      
 always @(posedge sysclk or negedge rst)
 begin 
   if(rst == low) 
    begin 
      cnt1 <= {N{1'b0}}; 
      rxclk <= hi; 
    end
   else if(cnt1 == (divp >> 1'b1))
    begin 
      cnt1 <= cnt1 + 1'b1; 
      rxclk <= low; 
    end
   else if(cnt1 == (divp - 1'b1))
    begin 
      cnt1 <= {N{1'b0}}; 
      rxclk <= hi; 
    end
   else 
    begin 
      cnt1 <= cnt1 + 1'b1; 
    end
 end 
             
/*     
 assign txclk = cnt2[3];        
       
 always @(posedge rxclk or negedge rst) begin 
   if(rst == low) begin 
     cnt2 <= 4'd0; 
   end
   else begin
     cnt2 <= cnt2 + 1'b1; 
   end
 end 
*/        
endmodule
