// ===========Oooo==========================================Oooo========
// =  Copyright (C) 2014 Jinan Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// =====================================================================
//
//  __      __      __
//  \ \    /  \    / /   [File name   ] fifo.v
//   \ \  / /\ \  / /    [Description ] Verilog File for UART Controller FIFO Module
//    \ \/ /  \ \/ /     [Timestamp   ] Tue Aug 27 15:30:00 2015
//     \  /    \  /      [version     ] 1.0
//      \/      \/       
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author |Mod. Date |Changes Made:
// V1.0 | XX     |27/08/15  |Initial version
// ===========Oooo==========================================Oooo========

module fifo_syn(
     datain  , 
     rd      ,
     wr      ,
     rst     ,
     clk     ,
     dataout ,
     full    ,
     empty
 );
 input  [7:0] datain  ; 
 input        rd      ;
 input        wr      ; 
 input        rst     ;
 input        clk     ;
 output [7:0] dataout ;  
 output       full    ;
 output       empty   ;
 
 reg    [7:0] dataout ;
 reg          full_in ;
 reg          empty_in; 
 reg [7:0] mem [15:0] ; 
 reg [3:0]    rp      ;
 reg [3:0]    wp      ;
 
 assign full = full_in; 
 assign empty = empty_in;   
 
 wire [31:8] mem_mem_0_0_DO;
 //
  SDP u_sdp_fifo8x16(
	.DO({mem_mem_0_0_DO[31:8], dataout[7:0]}),
	.DI({24'b0, datain[7:0]}),
	.ADA({7'b0,wp,3'b000}),
	.ADB({7'b0,rp,3'b000}),
	.CLKA(clk),
	.CLKB(clk),
	.WREA(wr && ~full_in),
	.WREB(1'b0),
	.OCE(1'b0),
	.CEA(1'b1),
	.CEB(rd && ~empty_in),
	.RESETA(!rst),
	.RESETB(!rst),
	.BLKSEL(3'b000)
 );
 defparam u_sdp_fifo8x16.READ_MODE=1'b0;
 defparam u_sdp_fifo8x16.BIT_WIDTH_0=8;
 defparam u_sdp_fifo8x16.BIT_WIDTH_1=8;
 defparam u_sdp_fifo8x16.BLK_SEL=3'b000;

 // memory write pointer increment 
 always@(posedge clk or negedge rst) 
 begin  
   if(rst == 0)      
     wp<=0;    
   else  if (wr && ~full_in)
     wp <= (wp + 1'b1) ;  
 end
 
 // memory read pointer increment 
 always@(posedge clk or negedge rst) 
 begin
   if(rst == 0)      
     rp <= 0;  
   else  
     rp <= (rd && ~empty_in)? (rp + 1'b1): rp;
  end
 
 // Full signal generate 
 always@(posedge clk or negedge rst) 
 begin    
   if(rst == 0)
     full_in <= 1'b0;   
   else 
   begin     
     if( (~rd && wr)&&((wp==rp-1)||(rp==4'h0&&wp==4'hf)))     
        full_in <= 1'b1;    
     else if(full_in && rd) 
        full_in <= 1'b0;   
   end 
 end   
   
 // Empty signal generate 
 always@(posedge clk or negedge rst) 
 begin 
   if(rst == 0) 
     empty_in <= 1'b1;
   else 
   begin    
     if((rd&&~wr)&&(rp==wp-1 || (rp==4'hf&&wp==4'h0)))      
        empty_in<=1'b1;  
     else if(empty_in && wr)
        empty_in<=1'b0;  
   end 
 end
endmodule
