
`timescale 1ns/1ps

`define C_S_AXI_DATA_WIDTH 32
`define C_S_AXI_ADDR_WIDTH 32
   
module master_axi_lite
(
	 S_AXI_ACLK,
     S_AXI_ARESETN,
     S_AXI_AWADDR,
     S_AXI_AWVALID,
     S_AXI_AWREADY,
     S_AXI_WDATA,
     S_AXI_WSTRB,
     S_AXI_WVALID,
     S_AXI_WREADY,
     S_AXI_BRESP,
     S_AXI_BVALID,
     S_AXI_BREADY,
     S_AXI_ARADDR,
     S_AXI_ARVALID,
     S_AXI_ARREADY,
     S_AXI_RDATA,
     S_AXI_RRESP,
     S_AXI_RVALID,
     S_AXI_RREADY,
     state_flag,
     error_flag
);

    input                                 S_AXI_ACLK;
    input                                 S_AXI_ARESETN;
    output   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_AWADDR;
    output                                S_AXI_AWVALID;
    input                                 S_AXI_AWREADY;
    output   [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_WDATA;
    output   [`C_S_AXI_DATA_WIDTH/8-1:0]  S_AXI_WSTRB;
    output                                S_AXI_WVALID;
    input                                 S_AXI_WREADY;
    input  [1:0]                          S_AXI_BRESP;
    input                                 S_AXI_BVALID;
    output                                S_AXI_BREADY;
    output   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_ARADDR;
    output                                S_AXI_ARVALID;
    input                                 S_AXI_ARREADY;
    input  [`C_S_AXI_DATA_WIDTH-1:0]      S_AXI_RDATA;
    input  [1:0]                          S_AXI_RRESP;
    input                                 S_AXI_RVALID;
    output                                S_AXI_RREADY;

    output reg                            state_flag    =   0;
    output reg                            error_flag    =   0;
 
 //////////////////////////////////////////////////////////////////////////
//	Internal Wires/Registers
    reg   [`C_S_AXI_ADDR_WIDTH - 1:0]       S_AXI_AWADDR;
    reg                                     S_AXI_AWVALID;

    reg   [`C_S_AXI_DATA_WIDTH-1:0]         S_AXI_WDATA;
    reg   [`C_S_AXI_DATA_WIDTH/8-1:0]       S_AXI_WSTRB;
    reg                                     S_AXI_WVALID;
 
    reg                                     S_AXI_BREADY;
    reg   [`C_S_AXI_ADDR_WIDTH - 1:0]       S_AXI_ARADDR;
    reg                                     S_AXI_ARVALID;

    reg                                     S_AXI_RREADY;
 
    reg								        axi_awready_reg;
    reg								        axi_arready_reg;
    reg								        axi_rvalid_reg;
 
    reg[3:0]							    wr_cntl;
    reg[3:0]							    wr_reg;
    reg[3:0]							    wr_index;
    reg[3:0]							    wr_rd_reg;
 
    reg[31:0]							    lsr_wr;
    reg[31:0]							    rd_data;
///////////////////////////////////////////////////////////////////////////

    always @(posedge S_AXI_ACLK)
    begin
	    if(~S_AXI_ARESETN)
		    axi_awready_reg <= 1'b0;
	    else
		    axi_awready_reg <= S_AXI_AWREADY;
    end

    always @(posedge S_AXI_ACLK)
    begin
	    if(~S_AXI_ARESETN)
		    axi_arready_reg <= 1'b0;
	    else
		    axi_arready_reg <= S_AXI_ARREADY;
    end

    always @(posedge S_AXI_ACLK)
    begin
	    if(~S_AXI_ARESETN)
		    axi_rvalid_reg <= 1'b0;
	    else
		    axi_rvalid_reg <= S_AXI_RVALID;
    end

    always @(posedge S_AXI_ACLK)
    begin
	    if(~S_AXI_ARESETN)
	    begin
		    S_AXI_AWADDR <= {`C_S_AXI_ADDR_WIDTH{1'b0}};
		    S_AXI_AWVALID <= 1'b0;
		    S_AXI_WDATA <= {`C_S_AXI_DATA_WIDTH{1'b0}};
		    S_AXI_WVALID <= 1'b0;
		    S_AXI_WSTRB <= {`C_S_AXI_DATA_WIDTH/8 {1'b0}};
		
		    S_AXI_ARVALID <= 1'b0;
		    S_AXI_RREADY <= 1'b0;
		    S_AXI_ARADDR <= {`C_S_AXI_ADDR_WIDTH{1'b0}};
		
		    wr_index <= 0;
		    wr_cntl <= 0;
		    wr_reg <=0;
		    wr_rd_reg <= 0;
		    lsr_wr <= 0;
            rd_data <= 0;
	    end
	else
	begin
		    if(wr_index==0)         // config lcr register
		    begin
		        case(wr_cntl)
		                        0:
		                            begin
		                            S_AXI_AWVALID <= 1'b1;
		                            S_AXI_WVALID <= 1'b1;
		                            S_AXI_AWADDR <= 32'h0000_000c;
		                            //S_AXI_WDATA <= 32'h0000_003F;
                                    S_AXI_WDATA <= 32'h0000_002B;   //32'h0000_001F;
                                    //S_AXI_WDATA <= 32'h0000_0003;
		                            S_AXI_WSTRB <= 4'b0001;
		
		                            wr_cntl <=1;
		                            end
		                        1:
		                            begin 
			                        if(S_AXI_AWREADY && ~axi_awready_reg)
			                            begin
				                            S_AXI_AWVALID <= 1'b0;
				                            S_AXI_WVALID <= 1'b0;
				                            S_AXI_AWADDR <= 0;
				                            S_AXI_WDATA <= 0;
				                            S_AXI_WSTRB <= 0;
			
				                            wr_cntl <=2;
			                            end
			                        else
			                            begin
				                            wr_cntl <=1;
			                            end	
		                            end
		                        2:
		                            begin
			                        wr_index <= 1;
			                        wr_cntl <=0;
		                            end
		            endcase 
		    end
		    else if(wr_index==1)    //write data (thr register)
		    begin
		            case(wr_reg)
		                        0:
		                            begin
		                            S_AXI_AWVALID <= 1'b1;
		                            S_AXI_WVALID <= 1'b1;
		                            S_AXI_AWADDR <= 32'h0000_0000;
		                            S_AXI_WDATA <= {24'h0000_00,8'h06};	//"5"
		                            //S_AXI_WDATA <= {24'h0000_00,8'ha1};	//"d"
		                            S_AXI_WSTRB <= 4'b0001;
		
		                            wr_reg <=1;
		                            end
		                        1:
		                            begin 
			                        if(S_AXI_AWREADY && ~axi_awready_reg)
			                            begin
				                        S_AXI_AWVALID <= 1'b0;
				                        S_AXI_WVALID <= 1'b0;
				                        S_AXI_AWADDR <= 0;
				                        S_AXI_WDATA <= 0;
				                        S_AXI_WSTRB <= 0;
			
				                        wr_reg <=2;
			                            end
			                        else
			                            begin
				                        wr_reg <=1;
			                            end	
		                            end
		                        2:
		                            begin
			                            wr_index <= 2;
			                            wr_reg <=0;
		                            end
		            endcase 
		    end
		    else if(wr_index==2)    //read data (rbr register)
			begin
			        case(wr_rd_reg)
                                0:
			                        begin
				                    S_AXI_ARVALID <= 1'b1;
				                    S_AXI_RREADY <= 1'b1;
				                    S_AXI_ARADDR <= 32'h0000_0000;
				
				                    wr_rd_reg <= 1;
			                        end
			                    1:
			                        begin
				                    if(S_AXI_ARREADY && ~axi_arready_reg)
                                        begin
					                        S_AXI_ARVALID <= 1'b0;
					
					                        wr_rd_reg <= 2; 
				                        end
				                    else 
                                        begin
					                    wr_rd_reg <= 1;
				                        end
			                        end
			                    2:
			                        begin
				                    if(S_AXI_RVALID && ~axi_rvalid_reg)
                                        begin
                                        state_flag <= 1'b1;
					                    S_AXI_RREADY <= 1'b0;
					                    rd_data <= S_AXI_RDATA;
                                        if((rd_data != 0) && (rd_data != 6))
                                            error_flag <= 1'b1;
					
					                    wr_rd_reg <= 0;
                                        wr_index <= 0;                    
				                        end
				                    else 
                                        begin
					                    wr_rd_reg <= 2;
				                        end
			                        end 
                        /*
			                    0:
			                        begin 
				                    S_AXI_ARVALID <= 1'b1;
				                    S_AXI_RREADY <= 1'b1;
				                    S_AXI_ARADDR <= 32'h0000_0014;
		
				                    wr_rd_reg <=1;
			                        end
			                    1:
			                        begin
				                    if(S_AXI_ARREADY && ~axi_arready_reg)
				                        begin
					                    S_AXI_ARVALID <= 1'b0;
					
					                    wr_rd_reg <= 2; 
				                        end
				                    else
				                        begin
					                    wr_rd_reg <= 1;
				                        end
			                        end
			                    2:
			                        begin
				                    if(S_AXI_RVALID && ~axi_rvalid_reg)
				                        begin
					                    S_AXI_RREADY <= 1'b0;
					                    lsr_wr <= S_AXI_RDATA;
					
					                    wr_rd_reg <= 3;					
				                        end
				                    else
				                        begin
					                    wr_rd_reg <= 2;
				                        end
			                        end	
			                    3:
			                        begin 
				                    if(lsr_wr[6]&&lsr_wr[5])
				                        begin
					                    wr_rd_reg <= 4;
				                        end
				                    else
				                        begin
					
					                    wr_rd_reg <= 0;
				                        end
			                        end
			                    4:
			                        begin 
				                    wr_index <= 0;
				                    wr_rd_reg <= 0;
			                        end
                                   */ 
			            endcase 
			    end 

	    end 												
    end

endmodule

