// ===========Oooo==========================================Oooo========
// =  Copyright (C) 2014 Jinan Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// =====================================================================
//
//  __      __      __
//  \ \    /  \    / /   [File name   ] modem.v
//   \ \  / /\ \  / /    [Description ] Verilog file for the Modem Interface
//    \ \/ /  \ \/ /     [Timestamp   ] Tue Aug 27 15:30:00 2015
//     \  /    \  /      [version     ] 1.0
//      \/      \/       
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author |Mod. Date |Changes Made:
// V1.0 | XX     |27/08/15  |Initial version
// ===========Oooo==========================================Oooo========
//
//    <Global reset and clock>
//      Reset      : Master reset
//      S_AXI_ACLK : System clock
//
//    <Registers>
//      MSR        : Modem Status Register
//      MCR        : Modem Control Register
//
//    <Rising edge of registers read/write strobes>
//      MsrRDn_re  : one S_AXI_ACLK width pulse indicating rising edge of MsrRDn_r
//
//    <Modem interface>
//      DCDn       : Data Carrier Detect
//      CTSn       : Clear To Send
//      DSRn       : Data Set Ready
//      RIn        : Ring Indicator
//      DTRn       : Data Terminal Ready
//      RTSn       : Request To Send
//
// ////////////////////////////////////////////////////////////////////

module Modem 
 (
    // Global reset and clock
    Reset      , // Master reset
    S_AXI_ACLK     , // UART internal clock
    // Registers
    MSR        , // Modem Status Reg
    MCR        , // Modem Control Reg
    // Rising Edge of MSR Read Strobe
    MsrRDn_re  , // pulse indicating rising of MsrRDn_r
    // Modem interface
    DCDn       , // Data Carrier Detect
    CTSn       , // Clear To Send
    DSRn       , // Data Set Ready
    RIn        , // Ring Indicator
    DTRn       , // Data Terminal Ready
    RTSn         // Request To Send
  );

input           Reset      ; // Master reset
input           S_AXI_ACLK     ; // UART internal clock
    // Registers
output [7:0]    MSR        ; // Modem Status Reg
input  [1:0]    MCR        ; // Modem Control Reg
    // Rising Edge of MSR Read Strobe
input           MsrRDn_re  ; // pulse indicating rising of MsrRDn_r
    // Modem interface
input           DCDn       ; // Data Carrier Detect
input           CTSn       ; // Clear To Send
input           DSRn       ; // Data Set Ready
input           RIn        ; // Ring Indicator
output          DTRn       ; // Data Terminal Ready
output          RTSn       ;  // Request To Send


  reg [7:0] MSReg   ;

  reg CTSn1   ; // Delayed version of CTSn
  reg DSRn1   ; // Delayed version of DSRn
  reg DCDn1   ; // Delayed version of DCDn
  reg RIn1    ; // Delayed version of RIn


   assign DTRn = ~ MCR[0];
   assign RTSn = ~ MCR[1];

////////////////////////////////////////////////////////////////////////////////
//      Modem Status Register Setup
////////////////////////////////////////////////////////////////////////////////

  assign MSR = MSReg;

//  Modem_Stat_Proc: 
always @(negedge Reset, posedge S_AXI_ACLK)
  begin
    if (Reset == 0) begin 
      MSReg <= 'b0;
      CTSn1 <= 1'b1;
      DSRn1 <= 1'b1;
      DCDn1 <= 1'b1;
      RIn1  <= 1'b1;
      end
    else begin
      CTSn1 <= CTSn; // Delay of CTSn
      DSRn1 <= DSRn; // Delay of DSRn
      DCDn1 <= DCDn; // Delay of DCDn
      RIn1 <= RIn;   // Delay of RIn
      if (MsrRDn_re==1'b1)
        MSReg <= 'b0;
      else begin
        MSReg[0] <= MSReg[0] | (CTSn1 ^ CTSn);     // Indicate DCDn changes
        MSReg[1] <= MSReg[1] | (DSRn1 ^ DSRn);     // Indicate DSRn changes
        MSReg[2] <= MSReg[2] | ((~RIn1) & RIn);    // Rising edge of RI
        MSReg[3] <= MSReg[3] | (DCDn1 ^ DCDn);     // Indicate DCDn changes
        MSReg[4] <= ~CTSn; // Compliment of CTSn
        MSReg[5] <= ~DSRn; // Compliment of DSRn
        MSReg[6] <= ~RIn;  // Compliment of RIn
        MSReg[7] <= ~DCDn; // Compliment of DCDn
        end
    end
  end 

endmodule


