
// ===========Oooo==========================================Oooo========
// =  Copyright (C) 2014-2017 Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// =====================================================================
//
//  __      __      __
//  \ \    /  \    / /   [File name   ] uart_slave_rx.v
//   \ \  / /\ \  / /    [Description ] UART Slave Model
//    \ \/ /  \ \/ /     [Timestamp   ] Wednesday September 04 10:00:30 2017
//     \  /    \  /      [version     ] 1.0.0
//      \/      \/       
//
// ===========Oooo==========================================Oooo========
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author |Mod. Date |Changes Made:
// V1.0 | XX     |09/13/17  |Initial version
// ===========Oooo==========================================Oooo========
`include    "uart_slave_defines.v"

`timescale  1ns / 1ps

module uart_slave_rx(
    sclk,
    rst_n,
    rx,
    rx_fram_end,
    data_rx
);

/************************************************
*ports and variables declaration
************************************************/
    input                           sclk;
    input                           rst_n;
    input                           rx;
    output reg                      rx_fram_end;
    output reg [`DATA_BIT-1:0]      data_rx;

    parameter                       POLARITY_BIT_NUM    =  `POLARITY_BIT ? 1 : 0;
    parameter                       STOP_BIT_NUM        =  (`STOP_BIT == 1 || `STOP_BIT == 3) ? 1 : 2;
    parameter                       FRAM_BIT_NUM        =  `START_BIT  +   `DATA_BIT   +   POLARITY_BIT_NUM    +    STOP_BIT_NUM;

    reg [FRAM_BIT_NUM-1:0]          shift_data_rx       =   0;
    reg                             div_end             =   0;
    reg [1:0]                       rx_reg              =   0;
    reg                             shift_flag          =   0;
    reg                             cnt_en              =   0;
    reg [23:0]                      div_cnt             =   0;
    reg [3:0]                       cnt_n               =   0;
 
/************************************************
*gen cnt_en:  rx(ing) data, cnt_en is 1
************************************************/ 
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            rx_reg <= 0;
        else
            rx_reg <= {rx_reg[0],rx};
    end

    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            cnt_en <= 0;
        else if(cnt_n == (FRAM_BIT_NUM-1) && div_cnt == (`BAUD_DIV-2))
            cnt_en <= 0;
        else if(rx_reg==2'b10)
            cnt_en <= 1;
    end

/**********************************************
*gen div_cnt: div_cnt counter
**********************************************/ 
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            div_cnt <= 0;
        else if(cnt_en == 0)
            div_cnt <= 0;
        else if(div_end == 1)
            div_cnt <= 0;
        else
            div_cnt <= div_cnt + 1;
    end

/********************************************
*gen div_end: count to BAUD_DIV
********************************************/
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            div_end <= 0;
        else if(div_cnt == `BAUD_DIV-2 /*|| (cnt_n == (FRAM_BIT_NUM-1) && (rx_reg == 2'b10))*/)
            div_end <= 1;
        else
            div_end <= 0;
    end

/*******************************************
*gen shift_flag: when BAUD_DIV/2 , start sample
*******************************************/
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            shift_flag <= 0;
        else if(cnt_en == 1 && div_cnt == `BAUD_DIV>>1)
            shift_flag <= 1;
        else
            shift_flag <= 0;
    end

    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            rx_fram_end<=0;
        else if(cnt_n == (FRAM_BIT_NUM-1) && div_end == 1'b1)
            rx_fram_end<=1;
        else
            rx_fram_end<=0;
    end

/******************************************
*gen cnt_n: sample counter
******************************************/
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            cnt_n <= 0;
        else if((cnt_en == 0) || (cnt_en == 1 && cnt_n == (FRAM_BIT_NUM-1) && div_end == 1'b1))
            cnt_n <= 0;
        else if(div_end == 1 && cnt_en == 1)
            cnt_n <= cnt_n + 1'b1;
    end

/*****************************************
*gen data_rx: save received data
*****************************************/ 
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            shift_data_rx <= 0;
        else if(shift_flag == 1)begin
            shift_data_rx <= {shift_data_rx[FRAM_BIT_NUM-2:0],rx};
        end
    end
 
    always@( posedge sclk or negedge rst_n)begin
        if(!rst_n)
            data_rx <= 0;
        else if(cnt_n == (FRAM_BIT_NUM-1) && div_end == 1'b1)begin
            if(`POLARITY_BIT == 0)begin
                if(`STOP_BIT == 1   ||  `STOP_BIT == 3)
                    data_rx <= shift_data_rx[`DATA_BIT+1:1];
                else if(`STOP_BIT == 2)
                    data_rx <= shift_data_rx[`DATA_BIT+2:2];
            end
            else begin
                if(`STOP_BIT == 1 || `STOP_BIT == 3)
                    data_rx <= shift_data_rx[`DATA_BIT+1:2];
                else if(`STOP_BIT == 2)
                    data_rx <= shift_data_rx[`DATA_BIT+2:3];
            end
        end
    end

endmodule