
// ===========Oooo==========================================Oooo========
// =  Copyright (C) 2014-2017 Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// =====================================================================
//
//  __      __      __
//  \ \    /  \    / /   [File name   ] uart_slave_tx.v
//   \ \  / /\ \  / /    [Description ] UART Slave Model
//    \ \/ /  \ \/ /     [Timestamp   ] Wednesday September 04 10:00:30 2017
//     \  /    \  /      [version     ] 1.0.0
//      \/      \/       
//
// ===========Oooo==========================================Oooo========
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author |Mod. Date |Changes Made:
// V1.0 | XX     |09/13/17  |Initial version
// ===========Oooo==========================================Oooo========
`include "uart_slave_defines.v"

`timescale  1 ns    /   1 ps

module uart_slave_tx(
    sclk,
    rst_n,
    data_tx,
    tx_fram_end,
    tx    
);
/********************************************
*ports and variables declaration
********************************************/
    input   sclk;
    input   rst_n;
    input [`DATA_BIT-1:0]   data_tx;
    input   tx_fram_end;
    output  tx;

    parameter               POLARITY_BIT_NUM    =  `POLARITY_BIT ? 1 : 0;
    parameter               STOP_BIT_NUM        =  (`STOP_BIT == 1 || `STOP_BIT == 3) ? 1 : 2;
    parameter               FRAM_BIT_NUM        =  `START_BIT  +   `DATA_BIT   +   POLARITY_BIT_NUM    +   STOP_BIT_NUM;


    reg                     div_end             =   0;
    reg                     busy_n              =   0;
    reg                     cnt_en              =   0;
    reg [3:0]               cnt_n               =   0;
    reg [23:0]              div_cnt             =   0;
    reg [FRAM_BIT_NUM-1:0]  shift_data          =   0;
    reg [3:0]               bit_num             =   0;
    reg                     shift_flag          =   0;
    reg                     tx_reg              =   1;
/*********************************************
*gen cnt_en
*********************************************/ 
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            cnt_en <= 0;
        else if(tx_fram_end == 1)
            cnt_en <= 1;
        else if((cnt_n == (FRAM_BIT_NUM-1)) && (div_end == 1))
            cnt_en <= 0;
    end

/**********************************************
*gen div_end: count to BAUD_DIV
**********************************************/ 
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            div_end <= 1'b0;
        else if( (cnt_en == 1) && (div_cnt == `BAUD_DIV-2))
            div_end <= 1'b1;
        else
            div_end <= 1'b0;
    end   

/*********************************************
*gen div_cnt
*********************************************/
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            div_cnt <= 0;
        else if(cnt_en == 0)
            div_cnt <= 0;
        else if(div_end == 1'b1)
            div_cnt <= 0;
        else
            div_cnt <= div_cnt + 1;
    end
/**********************************************
*gen cnt_n
**********************************************/     
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            cnt_n <= 0;
        else if( (tx_fram_end == 1) || (busy_n == 0) )
            cnt_n <= 0;
        else if(div_end == 1)
            cnt_n <= cnt_n + 1;
    end
/*******************************************
*gen shift_flag: when BAUD_DIV/2 , start sample
*******************************************/
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            shift_flag <= 0;
        else if(cnt_en == 1 && div_cnt == `BAUD_DIV>>1)
            shift_flag <= 1;
        else
            shift_flag <= 0;
    end
/**********************************************
*gen busy_n
**********************************************/ 
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            busy_n <= 0;
        else if(tx_fram_end == 1)
            busy_n <= 1;
        else if(cnt_n == (FRAM_BIT_NUM-1) && div_end == 1)
            busy_n <= 0;
    end
/**********************************************
*gen tx
**********************************************/
    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)
            shift_data <= 1;
        else if(tx_fram_end == 1 /*&& busy_n == 0*/)begin
            if(`POLARITY_BIT == 0) begin
                if(`STOP_BIT == 1   ||  `STOP_BIT == 3)
                    shift_data[FRAM_BIT_NUM-1:0] <= {1'b0,data_tx[`DATA_BIT-1:0],1'b1};
                else if(`STOP_BIT == 2)
                    shift_data[FRAM_BIT_NUM-1:0] <= {1'b0,data_tx[`DATA_BIT-1:0],1'b1,1'b1};
            end
            else begin
                if(`STOP_BIT == 1 ||    `STOP_BIT == 3)
                    shift_data[FRAM_BIT_NUM-1:0] <= {1'b0,data_tx[`DATA_BIT-1:0],1'b1,1'b1};
                else if(`STOP_BIT == 2)
                    shift_data[FRAM_BIT_NUM-1:0] <= {1'b0,data_tx[`DATA_BIT-1:0],1'b1,1'b1,1'b1};
            end
        end
    end

    always@(posedge sclk or negedge rst_n)begin
        if(!rst_n)begin
            bit_num <= FRAM_BIT_NUM-1;
            tx_reg <= 1;
        end
        else begin
            if(shift_flag)begin
                if(bit_num > 0)begin
                    tx_reg <= shift_data[bit_num];
                    bit_num <= bit_num - 1'b1;
                end
                else if(bit_num == 0)begin
                    tx_reg <= shift_data[bit_num];
                    bit_num <= FRAM_BIT_NUM - 1;
                end
            end
        end//else
    end

    assign tx = tx_reg;

endmodule